import 'package:chopper/chopper.dart';

import 'package:libaqueous/src/api/converter.dart';
import 'package:libaqueous/src/api/error_converter.dart';
import 'package:libaqueous/src/api/room/model/joined_rooms.dart';
import 'package:libaqueous/src/api/room/model/room_alias.dart';
import 'package:libaqueous/src/api/room/model/room_id.dart';
import 'package:libaqueous/src/api/room/room.dart';
import 'package:libaqueous/src/api/sync/model/sync_response.dart';
import 'package:libaqueous/src/api/sync/sync.dart';

class MatrixAPI {
  final ChopperClient chopper;

  final SyncAPI syncAPI;
  final RoomAPI roomAPI;

  final String baseUrl;
  final String user;
  final String token;

  MatrixAPI._fromAPI(this.chopper, this.syncAPI, this.roomAPI, this.baseUrl,
      this.user, this.token);

  factory MatrixAPI(String baseUrl, String user, String token) {
    final _chopper = ChopperClient(
      baseUrl: baseUrl,
      services: [
        SyncAPI.create(),
        RoomAPI.create(),
      ],
      converter: JsonSerializableConverter({
        JoinedRooms: JoinedRooms.fromJson,
        RoomID: RoomID.fromJson,
        RoomAlias: RoomAlias.fromJson,
        SyncResponse: SyncResponse.fromJson,
      }),
      errorConverter: ErrorConverter(),
      interceptors: [
        (Request request) async =>
            request.replace(headers: {"Authorization": "Bearer " + token}),
      ],
      jsonApi: true,
    );
    final _syncAPI = _chopper.service<SyncAPI>(SyncAPI);
    final _roomAPI = _chopper.service<RoomAPI>(RoomAPI);

    return MatrixAPI._fromAPI(
        _chopper, _syncAPI, _roomAPI, baseUrl, user, token);
  }
}
